package um.di.lmf.demo;

import um.di.lmf.security.*;

import iaik.asn1.*;
import iaik.security.rsa.*;
import iaik.x509.*;
import iaik.utils.*;
import java.math.*;
import java.util.*;
import java.io.*;
import java.security.*;
/**
 * Demonstration class for the RSA Blind Signature engine.
 * Look at the code
 */

public class RSABlindSignatureDemo {
    
    public static void main(String args[])  {
	try {
	    // Build the demo cert
	    ByteArrayInputStream bais = new ByteArrayInputStream(demo_cert.getBytes());
	    Base64InputStream b64is = new Base64InputStream(bais);
	    X509Certificate bank_cert=new X509Certificate(b64is);
	    RSAPublicKey bank_pub_key=new RSAPublicKey(bank_cert.getPublicKey().getEncoded());

	    // Build the demo private key
	    bais = new ByteArrayInputStream(demo_key.getBytes());
	    b64is = new Base64InputStream(bais);
	    byte[]key_bytes=new byte[bais.available()];
	    bais.read(key_bytes);
	    RSAPrivateKey bank_priv_key = RSAPrivateKey.parse(key_bytes);

	    // Get a good random source
	    SecureRandom sr=SecureRandom.getInstance("SHA1PRNG");
	    sr.setSeed(Calendar.getInstance().get(Calendar.MILLISECOND ));
		
	    // Generate a RSA key pair
	    //KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
	    //kpg.initialize(1024,sr);
	    //KeyPair kp = kpg.genKeyPair();
	    //RSAPublicKey bank_pub_key=new RSAPublicKey(kp.getPublic().getEncoded());
	    //RSAPrivateKey bank_priv_key=new RSAPrivateKey(kp.getPrivate().getEncoded());

	    // Generate a blinding factor relative prime to the modulus
	    BigInteger blinding_factor=new BigInteger(160,sr);
	    while (! blinding_factor.gcd(bank_pub_key.getModulus()).equals(BigInteger.ONE)) {
		blinding_factor=new BigInteger(160,sr);
	    }

	    //Generate something to sign
	    BigInteger anumber=new BigInteger(160,sr);

	    // Generate a blinded hash from an ASN1Object
	    byte[] blinded_hash=RSABlindSignature.blind(anumber.toByteArray(),blinding_factor,bank_pub_key);

	    // Sign the blinded hash
	    byte[] signature = RSABlindSignature.sign(blinded_hash, bank_priv_key);


	    // Unblind the signature
	    byte[] unb_sig=RSABlindSignature.unblind(signature, blinding_factor, bank_pub_key);

	    // Verify the unblinded signature
	    if(RSABlindSignature.verify(anumber.toByteArray(),unb_sig,bank_pub_key)) {
		System.out.println("Signature OK");
	    }
	    else {
		System.out.println("Signature Not OK");
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

static private String demo_cert ="MIICzzCCAjigAwIBAgIBHzANBgkqhkiG9w0BAQQFADBEMQswCQYDVQQGEwJQVDES\nMBAGA1UEChMJREktVU1JTkhPMQ8wDQYDVQQLEwZDQTAzMDQxEDAOBgNVBAMTB0NB\nLVJPT1QwHhcNMDMxMjAzMTIwNzA0WhcNMDQxMjAyMTIwNzA0WjBLMQswCQYDVQQG\nEwJQVDESMBAGA1UEChMJREktVU1JTkhPMQ8wDQYDVQQLEwZDQTAzMDQxFzAVBgNV\nBAMTDjE5Mi4xNjguMTAuMjM0MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCk\nKg6sRxOgKGXG5H6vzlxGAs0PFuPhNFWhz61A702Ntw8Qhq9nv512a7Im6OZS/5BF\n2qNjVvliM5Nr05wwdcEa+CRg/x3YieyKhIISYMfHyS/QhxtPGO0cYVyFsNgknafW\n8wNfoo8HJ4Vnr5t+eLubeHkg7A/166D4tJ4Aza7GXwIDAQABo4HJMIHGMAkGA1Ud\nEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5TU0wgR2VuZXJhdGVkIENlcnRpZmlj\nYXRlMB0GA1UdDgQWBBQAz3mXj8oIUxOBLs4LTkdnmSAsUTBsBgNVHSMEZTBjgBQr\n2lsBetN1DZ75kRV+vuey7J5GMaFIpEYwRDELMAkGA1UEBhMCUFQxEjAQBgNVBAoT\nCURJLVVNSU5ITzEPMA0GA1UECxMGQ0EwMzA0MRAwDgYDVQQDEwdDQS1ST09UggEA\nMA0GCSqGSIb3DQEBBAUAA4GBADzwudBv7bApnlS2Jzs/gXaZd01ExK85vOyANN9j\nxddWICHQIpb/UfLEEnjF6XmNLp91tKoGYH0jqmiS2/MfNwQ+fN4AlSxanhxF0qI8\nD2apTdiXSN/x16A1QBjJ8S4k0HUJJiw/B0q2zPwlJC3v5Nk8s3J1tiyoijNwhoDF\nkMVC";

    static private String demo_key = "MIICXgIBAAKBgQCkKg6sRxOgKGXG5H6vzlxGAs0PFuPhNFWhz61A702Ntw8Qhq9nv512a7Im6OZS/5BF2qNjVvliM5Nr05wwdcEa+CRg/x3YieyKhIISYMfHyS/QhxtP\nGO0cYVyFsNgknafW8wNfoo8HJ4Vnr5t+eLubeHkg7A/166D4tJ4Aza7GXwIDAQAB\nAoGBAI57VkreUcFS7qsGjJIoCFxXeATduCExXBVZ+6R9TpFuTavpSfpBC/jJFlHB\nGI728xA96EuDJGXAH/aZLlm9CkDq++AVCq49occh7EGvn/IwV05yTLiu8PoPDiEd\nP1hW5vqNt/BEy5946oPWI25gVyLDrloBe7eaScHIuQISAUdJAkEA1N8M3uB52LgL\nhZh9Q13Lz4Z2JNELWcmHeY4lVo7fIj+vZOoHDoW72/vm/f2R3YvPsStYL2nwxcB3\n3S46mdliUwJBAMVsuzVnGTExmB/D5oge9ta3afnTSmwuNk6F5u27zPe9WpEG1cKD\nxdzV1+m4cInciwGhpCN5u8v4i5iLu6hO4kUCQQCrfl7iF7pSLEOsB2OzKBeQQuVf\nV8JrPX6VIT1XdJazAteM1zWEL51e5pv7t5MSBxb1xtOurwy4ZAys9hdj5svdAkEA\nh/l1jzhtH3a7AdXZZ5N8CzLUaN7f+VZyzO5ouQCKEW93ez+r5ob/tjQKu8zupeUi\nUINWG6DyeW+8JdrH6wo5yQJAOXKYJnvqN8UKIC9cKJJJ4ALmBi5fsDHcF38JM0Wd\nCGrVUzQIQHwLYvqbMFiIllyyCBEWDUls8zfZXRnxOJvFZg==";
}