package um.di.lmf.enote;

import iaik.asn1.*;
import java.math.*;

import um.di.lmf.enote.*;

/**
 * This class represents a digital cash note 
 */

public class ENote extends Order implements ASN1Type {
    
    /**
     * This is the constructor that should be used before building an object with decode.
     */
    public ENote () {
    }

    /**
     * This constructor builds an eNote from an order and a signature.
     */
    public ENote (Order order, byte[] signature) {
	amount=order.amount;
	serial=order.serial;
	commit1=order.commit1;
	commit2=order.commit2;
	pub_random1=order.pub_random1;
	pub_random2=order.pub_random2;
	bank_cert=order.bank_cert;
	this.signature=signature;
    }

    /**
     * Returns the note signature
     */
    public byte[] getSignature() {
	return signature;
    }

    /** 
     * Decodes an ASN1Object according to the ASN.1 type the class implements.
     */
    public void decode(ASN1Object obj) throws CodingException {
	SEQUENCE enote = (SEQUENCE) obj;

	try {
	    super.decode(enote.getComponentAt(0));

	    signature=(byte []) ((OCTET_STRING) enote.getComponentAt(1)).getValue();
	
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    throw new CodingException();
	}
    }

    /**
     * Creates an ASN1Object according to the ASN.1 type the class implements.
     */
    public ASN1Object toASN1Object() throws CodingException {
	SEQUENCE enote=new SEQUENCE();

	enote.addComponent(super.toASN1Object());

        enote.addComponent(new OCTET_STRING(signature));

	return enote;
    }

    /**
     * The bank's signature
     */
    protected byte[]signature;


}