package um.di.lmf.enote;

import iaik.asn1.*;
import java.math.*;

import um.di.lmf.enote.*;

/**
 * This class represents a digital cash note disclosing half of the bit commitments for the payment protocol
 */

public class OpenENote extends ENote implements ASN1Type {
    
    /**
     * This is the constructor that should be used before building an object with decode.
     */
    public OpenENote () {
    }

    /**
     * This constructor builds an open enote from an enote and an open order
     */
    public OpenENote (OpenOrder oorder, ENote enote, boolean []bits) {
	super(oorder,enote.signature);
	this.bits=bits;
	for (int i=0; i< npairs; i++) {
	    idhalf[i]=oorder.getIDHalf(i,bits[i]);
	    priv_random[i]=oorder.getPrivRandom(i,bits[i]);
	}
    }

    /**
     * Returns one half of the ID
     */
    public byte[] getIDHalf(int i) {
	return idhalf[i];
    }

    /**
     * Returns one of the commitment secret randoms
     */
    public BigInteger getPrivRandom(int i) {
	return priv_random[i];
    }

    /**
     *
     */
    public boolean[] getBits() {
	return bits;
    }

    /** 
     * Decodes an ASN1Object according to the ASN.1 type the class implements.
     */
    public void decode(ASN1Object obj) throws CodingException {
	SEQUENCE oenote = (SEQUENCE) obj;

	try {
	    super.decode(oenote.getComponentAt(0));

	    idhalf=new byte[npairs][];
	    priv_random=new BigInteger[npairs];
	    
	    SEQUENCE pairs = (SEQUENCE) oenote.getComponentAt(1);
	    for (int i=0; i<npairs; i++) {
		SEQUENCE pair = (SEQUENCE) pairs.getComponentAt(i);
		idhalf[i]=(byte []) ((OCTET_STRING) pair.getComponentAt(0)).getValue();
		priv_random[i]=(BigInteger) ((INTEGER) pair.getComponentAt(1)).getValue();
	    };
	    String aux=(String) ((BIT_STRING) oenote.getComponentAt(2)).getBinaryString();
	    bits=new boolean[aux.length()];
	    for (int i=0;i<aux.length();i++){
		bits[i]=(aux.charAt(i)=='1');
	    }

	}
	catch (ArrayIndexOutOfBoundsException e) {
	    throw new CodingException();
	}
    }

   /**
     * Creates an ASN1Object of the enclosed ENote object
     */

    public ASN1Object toENoteASN1Object() throws CodingException {
	return super.toASN1Object();
    }

    /**
     * Creates an ASN1Object according to the ASN.1 type the class implements.
     */
    public ASN1Object toASN1Object() throws CodingException {
	SEQUENCE oenote=new SEQUENCE();

	oenote.addComponent(super.toASN1Object());

	SEQUENCE pairs = new SEQUENCE();
	for (int i=0; i<npairs; i++) {
	    SEQUENCE pair = new SEQUENCE();
	    pair.addComponent(new OCTET_STRING(idhalf[i]));
	    pair.addComponent(new INTEGER(priv_random[i]));
	    pairs.addComponent(pair);
	}
	oenote.addComponent(pairs);
	oenote.addComponent(new BIT_STRING(bits));

	return oenote;
    }

    /**
     * The private information for half of the bit commitments: id halves
     */
    protected byte[][] idhalf;

    /**
     * The private information for half of the bit commitments: private randoms
     */
    protected BigInteger[] priv_random;

    /**
     * The chosen half bits
     */

    protected boolean bits[];

}