package um.di.lmf.enote;

import iaik.asn1.*;
import java.math.*;
import java.security.cert.*;

import um.di.lmf.*;

/**
 * This class represents a digital cash note order, for the withdrawal protocol
 */

public class Order implements ASN1Type {
    
    /**
     * This is the constructor that should be used before building an object with decode.
     */
    public Order () {
    }

    /**
     * Returns the note money value
     */
    public int getAmount() {
	return amount;
    }

    /**
     * Returns the note serial number
     */
    public BigInteger getSerial() {
	return serial;
    }

   /**
     * Returns the number of id pairs
     */
    public int getNPairs() {
	return npairs;
    }

   /**
     * Returns one of the commitments
     */
    public byte[] getCommitment(int i,boolean bit) {
	if (!bit)
	    return commit1[i];
	else
	    return commit2[i];
    }

   /**
     * Returns one of the commitment public randoms
     */
    public BigInteger getPubRandom(int i,boolean bit) {
	if (!bit)
	    return pub_random1[i];
	else
	    return pub_random2[i];
    }

	/**
	* Returns the bank certificate
     */
    public iaik.x509.X509Certificate getBankCertificate() {
		return bank_cert;
    }
	
    /** 
     * Decodes an ASN1Object according to the ASN.1 type the class implements.
     */
    public void decode(ASN1Object obj) throws CodingException {
	SEQUENCE order = (SEQUENCE) obj;

	try {
	    amount=((BigInteger) ((INTEGER) order.getComponentAt(0)).getValue()).intValue();

	    serial=(BigInteger) ((INTEGER) order.getComponentAt(1)).getValue();

	    npairs=((BigInteger) ((INTEGER) order.getComponentAt(2)).getValue()).intValue();
	
	    commit1=new byte[npairs][];
	    commit2=new byte[npairs][];
	    pub_random1=new BigInteger[npairs];
	    pub_random2=new BigInteger[npairs];
	    
	    SEQUENCE pairs = (SEQUENCE) order.getComponentAt(3);
	    for (int i=0; i<npairs; i++) {
		SEQUENCE pair = (SEQUENCE) pairs.getComponentAt(i);
		commit1[i]=(byte []) ((OCTET_STRING) pair.getComponentAt(0)).getValue();
		commit2[i]=(byte []) ((OCTET_STRING) pair.getComponentAt(1)).getValue();
		pub_random1[i]=(BigInteger) ((INTEGER) pair.getComponentAt(2)).getValue();
		pub_random2[i]=(BigInteger) ((INTEGER) pair.getComponentAt(3)).getValue();
	    };
		
	    bank_cert=new iaik.x509.X509Certificate();
	    bank_cert.decode(order.getComponentAt(4));
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    throw new CodingException();
	}
    }

    /**
     * Creates an ASN1Object according to the ASN.1 type the class implements.
     */
    public ASN1Object toASN1Object() throws CodingException {
	SEQUENCE order=new SEQUENCE();

	order.addComponent(new INTEGER(amount));
	
	order.addComponent(new INTEGER(serial));

	order.addComponent(new INTEGER(npairs));
	
	SEQUENCE pairs = new SEQUENCE();
	for (int i=0; i<npairs; i++) {
	    SEQUENCE pair = new SEQUENCE();
	    pair.addComponent(new OCTET_STRING(commit1[i]));
	    pair.addComponent(new OCTET_STRING(commit2[i]));
	    pair.addComponent(new INTEGER(pub_random1[i]));
	    pair.addComponent(new INTEGER(pub_random2[i]));
	    pairs.addComponent(pair);
	}
	order.addComponent(pairs);

	order.addComponent(bank_cert.toASN1Object());

	return order;
    }

    /**
     * The money value of the ENote
     */
    protected int amount;
    
    /**
     * The serial number
     */
    protected BigInteger serial;

    /**
     * The number of IDPairs
     */
    protected int npairs;

    /**
     * The bit commitments
     */
    protected byte[][] commit1;

    /**
     * The bit commitments
     */
    protected byte[][] commit2;

    /**
     * The public random for the commitments
     */
    protected BigInteger[] pub_random1;

   /**
     * The public random for the commitments
     */
    protected BigInteger[] pub_random2;   
	
	/**
		* The bank's certificate
     */
    protected iaik.x509.X509Certificate bank_cert;

}